from typing import TYPE_CHECKING, Optional

from pydantic import PrivateAttr
from tygle.base import Chain
from tygle_sheets.types.enums import (
    DateTimeRenderOption,
    Dimension,
    InsertDataOption,
    ValueInputOption,
    ValueRenderOption,
)
from tygle_sheets.types.resources.values import ValueRange

if TYPE_CHECKING:
    from tygle_sheets.rest.values import Values


class ValuesChain(Chain):
    _rest: "Values" = PrivateAttr()
    spreadsheet_id: str

    def __init__(self, rest, **data):
        super().__init__(**data)
        self._rest = rest

    def get(
        self,
        range: str,
        /,
        *,
        major_dimension: Optional[Dimension] = None,
        value_render_option: Optional[ValueRenderOption] = None,
        date_time_render_option: Optional[DateTimeRenderOption] = None,
    ):
        return self._rest.get(
            self.spreadsheet_id,
            range,
            major_dimension=major_dimension,
            value_render_option=value_render_option,
            date_time_render_option=date_time_render_option,
        )

    def append(
        self,
        range: str,
        /,
        value_range: ValueRange,
        *,
        value_input_option: Optional[ValueInputOption] = None,
        insert_data_option: Optional[InsertDataOption] = None,
        include_values_in_response: Optional[bool] = None,
        response_value_render_option: Optional[ValueRenderOption] = None,
        response_date_time_render_option: Optional[DateTimeRenderOption] = None,
    ):
        return self._rest.append(
            self.spreadsheet_id,
            range,
            value_range,
            value_input_option=value_input_option,
            insert_data_option=insert_data_option,
            include_values_in_response=include_values_in_response,
            response_value_render_option=response_value_render_option,
            response_date_time_render_option=response_date_time_render_option,
        )



from typing import TYPE_CHECKING, ClassVar, List, Optional

from pydantic import Field, PrivateAttr
from tygle.base import Resource, RESTs

from .chains import ValuesChain
from .sheet import Sheet
from .spreadsheet_properties import SpreadsheetProperties

if TYPE_CHECKING:
    from tygle_sheets.rest.spreadsheets import Spreadsheets
    from tygle_sheets.rest.values import Values


class SpreadsheetRESTs(RESTs):
    def __init__(self, Spreadsheets: "Spreadsheets", Values: "Values"):
        self.Spreadsheets = Spreadsheets
        self.Values = Values


class Spreadsheet(Resource):
    """
    Resource that represents a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets#resource:-spreadsheet
    """

    __rests__: ClassVar[SpreadsheetRESTs] = PrivateAttr()

    @property
    def Values(self):
        return ValuesChain(self.__rests__.Values, spreadsheet_id=self.spreadsheet_id)

    def get(
        self,
        ranges: Optional[List[str]] = None,
        include_grid_data: bool = True,
    ):
        return self.__rests__.Spreadsheets.get(
            self.spreadsheet_id,
            ranges=ranges,
            include_grid_data=include_grid_data,
        )

    spreadsheet_id: str = Field(alias="spreadsheetId")
    properties: SpreadsheetProperties = Field()
    sheets: list[Sheet] = Field()



from typing import Optional

from pydantic import BaseModel, Field

from .error_value import ErrorValue


class ExtendedValue(BaseModel):
    """
    The kinds of value that a cell in a spreadsheet can have.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#ExtendedValue
    """

    number_value: Optional[float] = Field(None, alias="numberValue")
    string_value: Optional[str] = Field(None, alias="stringValue")
    bool_value: Optional[bool] = Field(None, alias="boolValue")
    formula_value: Optional[str] = Field(None, alias="formulaValue")
    error_value: Optional[ErrorValue] = Field(None, alias="errorValue")



from pydantic import BaseModel, Field

from .grid_data import GridData
from .grid_range import GridRange
from .sheet_properties import SheetProperties


class Sheet(BaseModel):
    """
    A sheet in a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#Sheet
    """

    properties: SheetProperties = Field()
    data: list[GridData] = Field()
    merges: list[GridRange] = Field()



from typing import Optional

from pydantic import BaseModel, Field


class GridRange(BaseModel):
    """
    A range on a sheet. All indexes are zero-based.
    Indexes are half open, i.e. the start index is inclusive and the end index is exclusive -- [startIndex, endIndex).
    Missing indexes indicate the range is unbounded on that side.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#GridRange
    """

    sheet_id: Optional[int] = Field(None, alias="sheetId")
    start_row_index: int = Field(alias="startRowIndex")
    end_row_index: int = Field(alias="endRowIndex")
    start_column_index: int = Field(alias="startColumnIndex")
    end_column_index: int = Field(alias="endColumnIndex")



from typing import Optional

from pydantic import BaseModel, Field

from .row_data import RowData


class GridData(BaseModel):
    """
    Data in the grid, as well as metadata about the dimensions.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#GridData
    """

    start_row: Optional[int] = Field(None, alias="startRow")
    start_column: Optional[int] = Field(None, alias="startColumn")
    row_data: list[RowData] = Field(alias="rowData")



from pydantic import BaseModel, Field

from .cell_data import CellData


class RowData(BaseModel):
    """
    Data about each cell in a row.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#RowData
    """

    values: list[CellData] = Field([])



from typing import TYPE_CHECKING, ClassVar, Optional

from pydantic import Field, PrivateAttr
from tygle.base import Resource, RESTs
from tygle_sheets.types.enums import (
    DateTimeRenderOption,
    Dimension,
    InsertDataOption,
    ValueInputOption,
    ValueRenderOption,
)

if TYPE_CHECKING:
    from tygle_sheets.rest.values import Values


class ValueRangeRESTs(RESTs):
    def __init__(self, Values: "Values"):
        self.Values = Values


class ValueRange(Resource):
    """Data within a range of the spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values#ValueRange
    """

    range: str = Field()
    major_dimension: Dimension = Field(alias="majorDimension")
    values: list[list] = Field()

    __rests__: ClassVar[ValueRangeRESTs] = PrivateAttr()

    def get(
        self,
        spreadsheet_id: str,
        /,
        *,
        major_dimension: Optional[Dimension] = None,
        value_render_option: Optional[ValueRenderOption] = None,
        date_time_render_option: Optional[DateTimeRenderOption] = None,
    ):
        return self.__rests__.Values.get(
            spreadsheet_id,
            self.range,
            major_dimension=major_dimension,
            value_render_option=value_render_option,
            date_time_render_option=date_time_render_option,
        )

    def append(
        self,
        spreadsheet_id: str,
        range: str,
        /,
        *,
        value_input_option: Optional[ValueInputOption] = None,
        insert_data_option: Optional[InsertDataOption] = None,
        include_values_in_response: Optional[bool] = None,
        response_value_render_option: Optional[ValueRenderOption] = None,
        response_date_time_render_option: Optional[DateTimeRenderOption] = None,
    ):
        """Chain to :meth:`.Values.append`."""
        return self.__rests__.Values.append(
            spreadsheet_id,
            range,
            self,
            value_input_option=value_input_option,
            insert_data_option=insert_data_option,
            include_values_in_response=include_values_in_response,
            response_value_render_option=response_value_render_option,
            response_date_time_render_option=response_date_time_render_option,
        )



# isort: skip_file

from .error_value import ErrorValue
from .extended_value import ExtendedValue
from .cell_data import CellData
from .row_data import RowData
from .grid_data import GridData
from .grid_range import GridRange
from .sheet_properties import SheetProperties
from .sheet import Sheet
from .spreadsheet_properties import SpreadsheetProperties
from .spreadsheet import Spreadsheet, SpreadsheetRESTs



from pydantic import BaseModel


class SpreadsheetProperties(BaseModel):
    """
    Properties of a spreadsheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets#spreadsheetproperties
    """

    title: str



from pydantic import BaseModel, Field


class SheetProperties(BaseModel):
    """
    Properties of a sheet.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/sheets#SheetProperties
    """

    sheet_id: int = Field(alias="sheetId")
    title: str = Field()



from pydantic import BaseModel, Field
from tygle_sheets.types.enums import ErrorType


class ErrorValue(BaseModel):
    """
    An error in a cell.

    https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/other#ErrorValue
    """

    type: ErrorType = Field()
    message: str = Field()



from typing import Optional

from aiogoogle import GoogleAPI
from pydantic import create_model
from tygle.base import REST
from tygle.base.requests import DataRequest
from tygle.client import Client
from tygle_sheets.types.enums import (
    DateTimeRenderOption,
    Dimension,
    InsertDataOption,
    ValueInputOption,
    ValueRenderOption,
)
from tygle_sheets.types.resources.values import ValueRange, ValueRangeRESTs
from tygle_sheets.types.responses.values import AppendValuesResponse


class Values(REST):
    def __init__(self, client: Client, parent: GoogleAPI) -> None:
        super().__init__(client, parent)

        self.ValueRange = create_model("ValueRange", __base__=ValueRange)
        self.ValueRange.__rests__ = ValueRangeRESTs(self)

    def get(
        self,
        spreadsheet_id: str,
        range: str,
        /,
        *,
        major_dimension: Optional[Dimension] = None,
        value_render_option: Optional[ValueRenderOption] = None,
        date_time_render_option: Optional[DateTimeRenderOption] = None,
    ) -> DataRequest["ValueRange"]:
        """Returns a range of values from a spreadsheet. The caller must specify the spreadsheet ID and a range.

        https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get
        """
        return DataRequest(
            self.client,
            self.parent.spreadsheets.values.get(
                spreadsheetId=spreadsheet_id,
                range=range,
                majorDimension=major_dimension.value if major_dimension else None,
                valueRenderOption=value_render_option.value
                if value_render_option
                else None,
                dateTimeRenderOption=date_time_render_option.value
                if date_time_render_option
                else None,
            ),
            self.ValueRange,
        )

    def append(
        self,
        spreadsheet_id: str,
        range: str,
        /,
        value_range: "ValueRange",
        *,
        value_input_option: Optional[ValueInputOption] = None,
        insert_data_option: Optional[InsertDataOption] = None,
        include_values_in_response: Optional[bool] = None,
        response_value_render_option: Optional[ValueRenderOption] = None,
        response_date_time_render_option: Optional[DateTimeRenderOption] = None,
    ) -> DataRequest[AppendValuesResponse]:
        """Appends values to a spreadsheet.

        https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append
        """
        return DataRequest(
            self.client,
            self.parent.spreadsheets.values.append(
                spreadsheetId=spreadsheet_id,
                range=range,
                json=value_range.dict(),
                valueInputOption=value_input_option.value
                if value_input_option
                else None,
                insertDataOption=insert_data_option.value
                if insert_data_option
                else None,
                includeValuesInResponse=include_values_in_response,
                responseValueRenderOption=response_value_render_option.value
                if response_value_render_option
                else None,
                responseDateTimeRenderOption=response_date_time_render_option.value
                if response_date_time_render_option
                else None,
            ),
            AppendValuesResponse,
        )
